import { useQuery } from '@tanstack/react-query';
import { fetchCodeFiles } from '@/api/code';

export function useCodeFiles(limit?: number, offset?: number) {
  return useQuery({
    queryKey: ['code', 'files', { limit, offset }],
    queryFn: () => fetchCodeFiles(limit, offset),
  });
}
