import { useQuery } from '@tanstack/react-query';
import { fetchCodeStats } from '@/api/code';

export function useCodeStats() {
  return useQuery({
    queryKey: ['code', 'stats'],
    queryFn: () => fetchCodeStats(),
  });
}
