import { useQuery } from '@tanstack/react-query';
import { fetchOtelStats } from '@/api/otel';

export function useOtelStats() {
  return useQuery({
    queryKey: ['otel', 'stats'],
    queryFn: () => fetchOtelStats(),
  });
}
