export const NODE_COLORS: Record<string, string> = {
  function: '#3b82f6',   // blue
  class: '#8b5cf6',      // violet
  method: '#10b981',     // green
  module: '#f59e0b',     // amber
  file: '#64748b',       // muted
  service: '#3b82f6',    // blue
  default: '#94a3b8',    // secondary
};

export const HEALTH_COLORS: Record<string, string> = {
  healthy: '#10b981',
  degraded: '#f59e0b',
  unhealthy: '#ef4444',
};

export const LAYOUT_SETTINGS = {
  iterations: 100,
  settings: {
    gravity: 1,
    scalingRatio: 2,
    barnesHutOptimize: true,
  },
};
