import { useQuery } from '@tanstack/react-query';
import { fetchOtelHealth } from '@/api/otel';

export function useOtelHealth() {
  return useQuery({
    queryKey: ['otel', 'health'],
    queryFn: () => fetchOtelHealth(),
  });
}
