import { useQuery } from '@tanstack/react-query';
import { fetchDocsStats } from '@/api/docs';

export function useDocsStats() {
  return useQuery({
    queryKey: ['docs', 'stats'],
    queryFn: () => fetchDocsStats(),
  });
}
