import { useQuery } from '@tanstack/react-query';
import { fetchDocuments } from '@/api/docs';

export function useDocuments(docType?: string, limit?: number, offset?: number) {
  return useQuery({
    queryKey: ['docs', 'documents', { docType, limit, offset }],
    queryFn: () => fetchDocuments(docType, limit, offset),
  });
}
