import { useQuery } from '@tanstack/react-query';
import { fetchMetrics } from '@/api/otel';

export function useMetrics(params?: Record<string, string | number | undefined>) {
  return useQuery({
    queryKey: ['otel', 'metrics', params],
    queryFn: () => fetchMetrics(params),
  });
}
