import { useQuery } from '@tanstack/react-query';
import { fetchDocsCommunities } from '@/api/docs';

export function useDocsCommunities() {
  return useQuery({
    queryKey: ['docs', 'communities'],
    queryFn: () => fetchDocsCommunities(),
  });
}
