import { useQuery } from '@tanstack/react-query';
import { fetchLogs } from '@/api/otel';

export function useLogs(params?: Record<string, string | number | undefined>) {
  return useQuery({
    queryKey: ['otel', 'logs', params],
    queryFn: () => fetchLogs(params),
  });
}
