import { useQuery } from '@tanstack/react-query';
import { fetchDocsEntities } from '@/api/docs';

export function useDocsEntities(entityName?: string, depth?: number, maxNodes?: number) {
  return useQuery({
    queryKey: ['docs', 'entities', { entityName, depth, maxNodes }],
    queryFn: () => fetchDocsEntities(entityName, depth, maxNodes),
  });
}
